package com.snucse.anjunbang.controller

import com.snucse.anjunbang.domain.AuctionState
import com.snucse.anjunbang.dto.AuctionDto
import com.snucse.anjunbang.service.auction.AuctionService
import com.snucse.anjunbang.service.bid.BidService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpHeaders
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.lang.Exception

@RestController
@RequestMapping("/api/v1/auctions")
class AuctionController(
        @Autowired private val auctionService: AuctionService,
        @Autowired private val bidService: BidService
) {
    @PostMapping("/create")
    fun createAuction(@RequestBody createAuctionReq: AuctionDto.CreateAuctionReq): ResponseEntity<AuctionDto.AuctionRes>{
        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(auctionService.createAuction(createAuctionReq))
    }

    @PostMapping("/complete/{id}")
    fun completeAuction(@PathVariable id: Long): ResponseEntity<Unit>{
        return if(auctionService.completeAuction(id)){
            ResponseEntity.ok().body(Unit)
        }
        else{
            ResponseEntity.badRequest().body(Unit)
        }
    }

    @GetMapping
    fun getAuctions(@RequestParam userId: Long?, @RequestParam isMyBid: Boolean, @RequestParam state: AuctionState?): ResponseEntity<List<AuctionDto.AuctionRes>>{
        var auctions: List<AuctionDto.AuctionRes> = listOf()

        // Case not available
        if(isMyBid && userId == null){
            throw Exception("BadRequest. It does not exist if the isMyBid field is true and userId is null.")
        }

        if(userId != null){
            // Case 1 : Group by User and isMyBid (내가 입찰한 경매)
            if(isMyBid){
                if(state == null){
                    auctions = bidService.getAuctionsByMyBid(userId)
                }
                else{
                    auctions = bidService.getAuctionsByMyBid(userId, state)
                }
            }
            // Case 2 : Group by User (나의 경매)
            else{
                if(state == null){
                    auctions = auctionService.getAuctionsByUser(userId)
                }
                else{
                    auctions = auctionService.getAuctionsByUser(userId, state)
                }
            }
        }
        // Case 3 : Default (전체 경매)
        else{
            if(state == null){
                auctions = auctionService.getAuctions()
            }
            else{
                auctions = auctionService.getAuctions(state)
            }
        }

        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(auctions);
    }
}