package com.snucse.anjunbang.controller

import com.snucse.anjunbang.dto.ProductDto
import com.snucse.anjunbang.service.product.ProductService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpHeaders
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("api/v1/products")
class ProductController(
        @Autowired private val productService: ProductService
) {
    @PostMapping("/create")
    fun createProduct(@RequestBody createProductReq: ProductDto.CreateProductReq): ResponseEntity<ProductDto.ProductRes>{
        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(productService.createProduct(createProductReq))
    }

    @PostMapping("/upload/image", consumes = [ MediaType.MULTIPART_FORM_DATA_VALUE ])
    fun createProductImage(uploadImageReq: ProductDto.UploadImageReq): ResponseEntity<ProductDto.UploadImageRes>{
        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(productService.uploadImage(uploadImageReq))
    }

    @GetMapping
    fun getProducts(@RequestParam userId: Long?): ResponseEntity<List<ProductDto.ProductRes>>{
        var products = listOf<ProductDto.ProductRes>()

        if(userId != null){
            products = productService.getProductsByUser(userId)
        }
        else{
            products = productService.getProducts()
        }

        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(products)
    }

    @GetMapping("/{id}")
    fun getProduct(@PathVariable id: Long): ResponseEntity<ProductDto.ProductRes>{
        return ResponseEntity
                .ok()
                .header(HttpHeaders.CONTENT_TYPE, "application/json")
                .body(productService.getProduct(id))
    }
}