package com.snucse.anjunbang.domain

import com.fasterxml.jackson.annotation.JsonIgnore
import com.snucse.anjunbang.dto.AuctionDto
import java.time.LocalDateTime
import javax.persistence.*

@Entity
@Table(name = "auction")
data class Auction(
        @Id @GeneratedValue(strategy = GenerationType.IDENTITY)
        var id: Long? = null,

        var title: String,
        var description: String,

        var startAt: LocalDateTime,
        var endAt: LocalDateTime,

        @ManyToOne(fetch = FetchType.LAZY)
        @JoinColumn(name = "user_id")
        var user: User,

        @ManyToOne(fetch = FetchType.LAZY)
        @JoinColumn(name = "product_id")
        var product: Product,

        @OneToMany(mappedBy = "auction")
        var bids: MutableList<Bid> = mutableListOf(),

        @Enumerated(EnumType.STRING)
        var auctionState: AuctionState? = AuctionState.PROCESSING
){
    companion object{
        fun createAuction(createAuctionReq: AuctionDto.CreateAuctionReq, user: User, product: Product) = Auction(
                title = createAuctionReq.title,
                description = createAuctionReq.description,
                startAt = createAuctionReq.startAt,
                endAt = createAuctionReq.endAt,
                user = user,
                product = product
        )
    }

    fun toAuctionRes() = AuctionDto.AuctionRes(
            auctionId = id!!,
            title = title,
            description = description,
            startAt = startAt.toString(),
            endAt = endAt.toString(),
            product = product.toProductRes(),
            bids = bids.map { bid -> bid.toBidRes() }.sortedByDescending { bid -> bid.price }.distinct(),
            auctionState = auctionState
    )
}

enum class AuctionState{
    PROCESSING, COMPLETED
}