package com.snucse.anjunbang.domain

import com.snucse.anjunbang.dto.ProductDto
import javax.persistence.*

@Entity
@Table(name = "product")
data class Product (
        @Id @GeneratedValue(strategy =  GenerationType.IDENTITY)
        var id: Long? = null,

        @Column(name = "name", length = 20)
        var name: String,

        @Column(name = "address", length = 40)
        var address: String,

        @Column(name = "detail_address", length = 40)
        var detailAddress: String,

        @OneToMany(mappedBy = "product")
        var productImages: MutableList<ProductImage> = mutableListOf(),

        @ManyToOne(fetch = FetchType.LAZY)
        @JoinColumn(name = "user_id")
        var user: User,

        @Column(name = "area")
        var area: Double,

        @Column(name = "room_architecture", length = 40)
        var roomArchitecture: String,

        @Column(name = "building_format", length = 40)
        var buildingFormat: String,

        @Column(name = "description", length = 40)
        var description: String
) {
        companion object{
                fun createProduct(createProductReq: ProductDto.CreateProductReq, user: User, productImages: MutableList<ProductImage>) = Product(
                        name = createProductReq.name,
                        productImages = productImages,
                        address = createProductReq.address,
                        detailAddress = createProductReq.detailAddress,
                        area = createProductReq.area,
                        roomArchitecture = createProductReq.roomArchitecture,
                        buildingFormat = createProductReq.buildingFormat,
                        description = createProductReq.description,
                        user = user
                )
        }

        fun toProductRes() = ProductDto.ProductRes(
                productId = id!!,
                name = name,
                imageUrls = productImages.map { productImage -> productImage.imageUrl },
                address = address,
                detailAddress = detailAddress,
                area = area,
                roomArchitecture = roomArchitecture,
                buildingFormat = buildingFormat,
                description = description,
                user = user.toUserRes()
        )
}