package com.snucse.anjunbang.service.auction

import com.snucse.anjunbang.domain.Auction
import com.snucse.anjunbang.domain.AuctionState
import com.snucse.anjunbang.dto.AuctionDto
import com.snucse.anjunbang.exception.AuctionNotFoundException
import com.snucse.anjunbang.repository.AuctionRepository
import com.snucse.anjunbang.service.bid.BidService
import com.snucse.anjunbang.service.product.ProductService
import com.snucse.anjunbang.service.user.UserService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.lang.Exception
import javax.transaction.Transactional

@Service
@Transactional
class AuctionServiceImpl(
        @Autowired private val auctionRepository: AuctionRepository,
        @Autowired private val userService: UserService,
        @Autowired private val productService: ProductService,
        @Autowired private val bidService: BidService
): AuctionService {
    override fun createAuction(createAuctionReq: AuctionDto.CreateAuctionReq): AuctionDto.AuctionRes{
        val user = userService.getUserById(createAuctionReq.userId)
        val product = productService.getProductById(createAuctionReq.productId)
        val auction = auctionRepository.save(Auction.createAuction(createAuctionReq, user, product))

        return auction.toAuctionRes()
    }

    override fun completeAuction(id: Long): Boolean{
        var isSuccess = false
        try{
            val auction = auctionRepository.findById(id).orElseThrow { throw AuctionNotFoundException("AuctionNotFoundException. Cannot find auction by id: $id.") }
            auction.auctionState = AuctionState.COMPLETED
            isSuccess = true
        }
        catch (e: Exception){
            // Do somethings...
        }
        return isSuccess;
    }

    override fun getAuctions(): List<AuctionDto.AuctionRes>{
        val auctions = auctionRepository.findAll()
        return auctions.map { auction -> auction.toAuctionRes() }
    }

    override fun getAuctions(state: AuctionState): List<AuctionDto.AuctionRes> {
        val auctions = auctionRepository.findAuctionsByAuctionState(state)
        return auctions.map { auction -> auction.toAuctionRes() }
    }

    override fun getAuctionsByUser(userId: Long): List<AuctionDto.AuctionRes>{
        val auctions = auctionRepository.findAuctionsByUser_Id(userId)
        return auctions.map { auction -> auction.toAuctionRes() }
    }

    override fun getAuctionsByUser(userId: Long, state: AuctionState): List<AuctionDto.AuctionRes>{
        val auctions = auctionRepository.findAuctionsByUser_IdAndAuctionState(userId, state)
        return auctions.map { auction -> auction.toAuctionRes() }
    }

//    override fun getAuctionsByUserBid(userId: Long): List<AuctionDto.AuctionRes>{
//        var bids = bidService.getBidsByUser(userId)
//        bids = bids.filter { bid -> bid.user.userId == userId }
//        return bids.map { bid -> bid.auction }
//    }

//    override fun getAuctionsByUserBid(userId: Long, state: AuctionState): List<AuctionDto.AuctionRes>{
//        var bids = bidService.getBidsByUser(userId)
//        bids = bids.filter { bid -> bid.auction.auctionState == state }
//        return bids.map { bid -> bid.auction }
//    }

    override fun getAuction(id: Long): AuctionDto.AuctionRes{
        val auction = auctionRepository.findById(id).orElseThrow { throw AuctionNotFoundException("AuctionNotFoundException. Cannot find auction by id: $id.") }
        return auction.toAuctionRes()
    }

    override fun getAuctionById(id: Long): Auction =
            auctionRepository.findById(id).orElseThrow { throw AuctionNotFoundException("AuctionNotFoundException. Cannot find user by id: $id.") }
}