package com.snucse.anjunbang.service.bid

import com.snucse.anjunbang.domain.AuctionState
import com.snucse.anjunbang.domain.Bid
import com.snucse.anjunbang.dto.AuctionDto
import com.snucse.anjunbang.dto.BidDto
import com.snucse.anjunbang.exception.AuctionNotFoundException
import com.snucse.anjunbang.exception.BidNotFoundException
import com.snucse.anjunbang.repository.AuctionRepository
import com.snucse.anjunbang.repository.BidRepository
import com.snucse.anjunbang.service.user.UserService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import javax.transaction.Transactional

@Service
@Transactional
class BidServiceImpl(
        @Autowired private val bidRepository: BidRepository,
        @Autowired private val userService: UserService,
        @Autowired private val auctionRepository: AuctionRepository
): BidService {

    override fun createBid(createBidReq: BidDto.CreateBidReq): BidDto.BidRes{
        val user = userService.getUserById(createBidReq.userId)
        val auction = auctionRepository.findById(createBidReq.auctionId).orElseThrow { throw AuctionNotFoundException("AuctionNotFoundException. Cannot find auction by id : ${createBidReq.auctionId}") }
        val bid = bidRepository.save(Bid.createBid(createBidReq, user, auction))
        return bid.toBidRes()
    }

    override fun getBids(): List<BidDto.BidRes>{
        val bids = bidRepository.findAll()
        return bids.map { bid -> bid.toBidRes() }
    }

    override fun getAuctionsByMyBid(userId: Long): List<AuctionDto.AuctionRes>{
        val bids = bidRepository.findBidsByUser_Id(userId)
        return bids.map { bid -> bid.auction.toAuctionRes() }
    }

    override fun getAuctionsByMyBid(userId: Long, state: AuctionState): List<AuctionDto.AuctionRes>{
        val bids = bidRepository.findBidsByUser_Id(userId)
        return bids.map { bid -> bid.auction.toAuctionRes() }.filter { auction -> auction.auctionState == state }
    }

    override fun getBidsByUser(userId: Long): List<BidDto.BidRes>{
        val bids = bidRepository.findBidsByUser_Id(userId)
        return bids.map { bid -> bid.toBidRes() }
    }

    override fun getBidsByAuction(auctionId: Long): List<BidDto.BidRes>{
        val bids = bidRepository.findBidsByAuction_Id(auctionId)
        return bids.map { bid -> bid.toBidRes() }
    }

    override fun getBidsByUserAndAuction(userId: Long, auctionId: Long): List<BidDto.BidRes>{
        val bids = bidRepository.findBidsByUser_IdAndAuction_Id(userId, auctionId)
        return bids.map { bid -> bid.toBidRes() }
    }

    override fun getBid(id: Long): BidDto.BidRes{
        val bid = bidRepository.findById(id).orElseThrow { throw BidNotFoundException("BidNotFoundException. Cannot find bid by id: $id.") }
        return bid.toBidRes()
    }
}