package com.snucse.anjunbang.service.product

import com.amazonaws.services.s3.AmazonS3
import com.snucse.anjunbang.domain.Product
import com.snucse.anjunbang.domain.ProductImage
import com.snucse.anjunbang.dto.ProductDto
import com.snucse.anjunbang.exception.ImageUploadException
import com.snucse.anjunbang.exception.ProductNotFoundException
import com.snucse.anjunbang.repository.ProductRepository
import com.snucse.anjunbang.service.product.image.ProductImageService
import com.snucse.anjunbang.service.user.UserService
import com.snucse.anjunbang.util.S3Uploader
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import javax.transaction.Transactional

@Service
@Transactional
class ProductServiceImpl(
        @Autowired private val productRepository: ProductRepository,
        @Autowired private val amazonS3Client: AmazonS3,
        @Autowired private val userService: UserService,
        @Autowired private val productImageService: ProductImageService
): ProductService {

    override fun createProduct(createProductReq: ProductDto.CreateProductReq): ProductDto.ProductRes {
        val user = userService.getUserById(createProductReq.userId)

        var productImages: MutableList<ProductImage> = mutableListOf()

        if(createProductReq.imageUrls != null && createProductReq.imageUrls!!.count() > 0){
            productImages = productImageService.createProductImages(createProductReq.imageUrls!!)
        }

        val product = productRepository.save(Product.createProduct(createProductReq, user, productImages))

        productImages.map { productImage -> productImage.product = product }

        return product.toProductRes()
    }

    override fun getProducts(): List<ProductDto.ProductRes>{
        val products = productRepository.findAll()
        return products.map { product -> product.toProductRes() }
    }

    override fun getProductsByUser(userId: Long): List<ProductDto.ProductRes>{
        val products = productRepository.findProductByUser_Id(userId)
        return products.map { product -> product.toProductRes() }
    }

    override fun getProduct(id: Long): ProductDto.ProductRes{
        val product = productRepository.findById(id).orElseThrow { throw ProductNotFoundException("ProductNotFoundException. Cannot find product by id: $id.") }
        return product.toProductRes()
    }

    override fun getProductById(id: Long): Product =
            productRepository.findById(id).orElseThrow { throw ProductNotFoundException("ProductNotFoundException. Cannot find product by id: $id.") }

    override fun uploadImage(uploadImageReq: ProductDto.UploadImageReq): ProductDto.UploadImageRes{
        if(uploadImageReq.image == null){
            return ProductDto.UploadImageRes(
                    imageUrl = ""
            )
        }

        val s3Uploader = S3Uploader(amazonS3Client)

        try{
            var imageUrl: String = s3Uploader.upload(uploadImageReq.image!!, "anjun-bang.static.io", "products") ?: throw ImageUploadException("failed to upload product's image to s3.")
            imageUrl = imageUrl.replace("https", "http")
            return ProductDto.UploadImageRes(imageUrl = imageUrl)
        }
        catch (e: Exception){
            throw e
        }
    }
}