package com.snucse.anjunbang.service.user

import com.snucse.anjunbang.domain.User
import com.snucse.anjunbang.dto.UserDto
import com.snucse.anjunbang.exception.DuplicateEmailException
import com.snucse.anjunbang.exception.DuplicateNicknameException
import com.snucse.anjunbang.exception.UserNotFoundException
import com.snucse.anjunbang.exception.WrongPasswordException
import com.snucse.anjunbang.repository.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.stereotype.Service
import javax.transaction.Transactional

@Service
@Transactional
class UserServiceImpl(
        @Autowired private val userRepository: UserRepository,
        @Autowired private val bCryptPasswordEncoder: BCryptPasswordEncoder
): UserService {

    override fun signUp(signUpReq: UserDto.SignUpReq): UserDto.UserRes{
        if(userRepository.existsByEmail(signUpReq.email))
            throw DuplicateEmailException("DuplicateEmailException. Email: ${signUpReq.email}")

        if(userRepository.existsByNickname(signUpReq.nickname))
            throw DuplicateNicknameException("DuplicateEmailException. Nickname: ${signUpReq.nickname}")

        val user = userRepository.save(signUpReq.toEntity(bCryptPasswordEncoder))

        return user.toSignUpRes()
    }

    override fun signIn(signInReq: UserDto.SignInReq): UserDto.UserRes{
        val user = getUserByEmail(signInReq.email)

        if(!user.isRightPassword(bCryptPasswordEncoder, signInReq.password))
            throw WrongPasswordException("WrongPasswordException.")

        return user.toSignInRes()
    }

    override fun getUserById(id: Long): User =
            userRepository.findById(id).orElseThrow { throw UserNotFoundException("UserNotFoundException. Cannot find user by id: $id.")}

    fun getUserByEmail(email: String): User =
            userRepository.findByEmail(email) ?: throw UserNotFoundException("UserNotFoundException. Cannot find user by email: $email")
}